/*
 * Common system pager functionality.
 *
 * Copyright (C) 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <map>

#include <exec/mapped_region.h>



/* Collection types. */

typedef std::map<l4_addr_t, MappedRegion> MappedRegions;



/* A simple system pager also acting as a region mapper. */

class ExecPager
{
protected:
  address_t _start, _end;

  /* Region manager/mapper functionality. */

  MappedRegions _regions;

  virtual long find(address_t *start, address_t *size, map_flags_t flags,
                    address_t offset, unsigned char align);

public:
  explicit ExecPager(address_t start = 0, address_t end = 0);

  virtual ~ExecPager();

  /* Region management methods. */

  virtual void add(MappedRegion &region);

  /* Notification methods. */

  virtual long exception(l4_exc_regs_t regs,
                         l4_snd_fpage_t *region) = 0;

  virtual long page_fault(l4_umword_t pfa, l4_umword_t pc,
                          l4_snd_fpage_t *region) = 0;
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
